@@ -6,19 +6,19 @@
 
 
 """ 封装一些代码开发中常用的功能，工程组件 """
-import itertools
-from urllib.parse import urlparse
+from collections import defaultdict
 import io
+import itertools
 import json
 import math
 import os
 import queue
 import socket
+import subprocess
 import sys
-import time
-import pprint
 import tempfile
-from functools import partial
+import time
+from urllib.parse import urlparse
 
 from pyxllib.prog.newbie import classproperty
 
@@ -241,6 +241,85 @@ class DictTool:
                 del dict_[k]
 
 
+class EnchantCvt:
+    """ 把类_cls的功能绑定到类cls里
+
+    根源_cls里的实现类型不同，到cls需要呈现的接口形式不同，有很多种不同的转换形式
+
+    每个分支里，随附了getattr目标函数的一般默认定义模板
+    用_self、_cls表示dst_cls，区别原cls类的self、cls标记
+    """
+
+    @staticmethod
+    def staticmethod2objectmethod(cls, _cls, x):
+        # 目前用的最多的转换形式
+        # @staticmethod
+        # def func1(_self, *args, **kwargs): ...
+        setattr(_cls, x, getattr(cls, x))
+
+    @staticmethod
+    def staticmethod2property(cls, _cls, x):
+        # @staticmethod
+        # def func2(_self): ...
+        setattr(_cls, x, property(getattr(cls, x)))
+
+    @staticmethod
+    def staticmethod2classmethod(cls, _cls, x):
+        # @staticmethod
+        # def func3(_cls, *args, **kwargs): ...
+        setattr(_cls, x, classmethod(getattr(cls, x)))
+
+    @staticmethod
+    def staticmethod2classproperty(cls, _cls, x):
+        # @staticmethod
+        # def func4(_cls): ...
+        setattr(_cls, x, classproperty(getattr(cls, x)))
+
+    @staticmethod
+    def classmethod2objectmethod(cls, _cls, x):
+        # @classmethod
+        # def func5(cls, _self, *args, **kwargs): ...
+        setattr(_cls, x, lambda *args, **kwargs: getattr(cls, x)(*args, **kwargs))
+
+    @staticmethod
+    def classmethod2property(cls, _cls, x):
+        # @classmethod
+        # def func6(cls, _self): ...
+        setattr(_cls, x, lambda *args, **kwargs: property(getattr(cls, x)(*args, **kwargs)))
+
+    @staticmethod
+    def classmethod2classmethod(cls, _cls, x):
+        # @classmethod
+        # def func7(cls, _cls, *args, **kwargs): ...
+        setattr(_cls, x, lambda *args, **kwargs: classmethod(getattr(cls, x)(*args, **kwargs)))
+
+    @staticmethod
+    def classmethod2classproperty(cls, _cls, x):
+        # @classmethod
+        # def func8(cls, _cls): ...
+        setattr(_cls, x, lambda *args, **kwargs: classproperty(getattr(cls, x)(*args, **kwargs)))
+
+    @staticmethod
+    def staticmethod2modulefunc(cls, _cls, x):
+        # @staticmethod
+        # def func9(*args, **kwargs): ...
+        setattr(_cls, x, getattr(cls, x))
+
+    @staticmethod
+    def classmethod2modulefunc(cls, _cls, x):
+        # @classmethod
+        # def func10(cls, *args, **kwargs): ...
+        setattr(_cls, x, lambda *args, **kwargs: getattr(cls, x)(*args, **kwargs))
+
+    @staticmethod
+    def to_moduleproperty(cls, _cls, x):
+        # 理论上还有'to_moduleproperty'的转换模式
+        #   但这个很容易引起歧义，是应该存一个数值，还是动态计算？
+        #   如果是动态计算，可以使用modulefunc的机制显式执行，更不容易引起混乱。
+        #   从这个分析来看，是不需要实现'2moduleproperty'的绑定体系的。py标准语法本来也就没有module @property的概念。
+        raise NotImplementedError
+
+
 class EnchantBase:
     """
     一些三方库的类可能功能有限，我们想做一些扩展。
@@ -256,12 +335,13 @@ class EnchantBase:
     """
 
     @classmethod
-    def check_enchant_names(cls, classes, names=None, *, white_list=None):
+    def check_enchant_names(cls, classes, names=None, *, white_list=None, ignore_case=False):
         """
         :param list classes: 不能跟这里列出的模块、类的成员重复
         :param list|str|tuple names: 要检查的名称清单
         :param white_list: 白名单，这里面的名称不警告
             在明确要替换三方库标准功能的时候，可以使用
+        :param ignore_case: 忽略大小写
         """
         exist_names = {x.__name__: set(dir(x)) for x in classes}
         if names is None:
@@ -269,6 +349,13 @@ class EnchantBase:
                     - {'check_enchant_names', '_enchant', 'enchant'}
 
         white_list = set(white_list) if white_list else {}
+
+        if ignore_case:
+            names = {x.lower() for x in names}
+            for k, values in exist_names.items():
+                exist_names[k] = {x.lower() for x in exist_names[k]}
+            white_list = {x.lower() for x in white_list}
+
         for name, k in itertools.product(names, exist_names):
             if name in exist_names[k] and name not in white_list:
                 print(f'警告！同名冲突！ {k}.{name}')
@@ -276,62 +363,55 @@ class EnchantBase:
         return set(names)
 
     @classmethod
-    def _enchant(cls, _cls, names=None, mode='staticmethod2objectmethod', *, white_list=None):
+    def _enchant(cls, _cls, names, cvt=EnchantCvt.staticmethod2objectmethod):
         """ 这个框架是支持classmethod形式的转换的，但推荐最好还是用staticmethod，可以减少函数嵌套层数，提高效率 """
-        names = cls.check_enchant_names([_cls], names, white_list=white_list)
-
-        # 每个if分支里，随附了getattr目标函数的一般默认定义模板
-        # 用_self、_cls表示dst_cls，区别原cls类的self、cls标记
-        if mode in 'staticmethod2objectmethod':  # 目前用的最多的转换形式
-            # @staticmethod
-            # def func1(_self, *args, **kwargs): ...
-            func = lambda x: setattr(_cls, x, getattr(cls, x))
-        elif mode == 'staticmethod2property':
-            # @staticmethod
-            # def func2(_self): ...
-            func = lambda x: setattr(_cls, x, property(getattr(cls, x)))
-        elif mode == 'staticmethod2classmethod':
-            # @staticmethod
-            # def func3(_cls, *args, **kwargs): ...
-            func = lambda x: setattr(_cls, x, classmethod(getattr(cls, x)))
-        elif mode == 'staticmethod2classproperty':
-            # @staticmethod
-            # def func4(_cls): ...
-            func = lambda x: setattr(_cls, x, classproperty(getattr(cls, x)))
-        elif mode == 'classmethod2objectmethod':
-            # @classmethod
-            # def func5(cls, _self, *args, **kwargs): ...
-            func = lambda x: setattr(_cls, x, lambda *args, **kwargs: getattr(cls, x)(*args, **kwargs))
-        elif mode == 'classmethod2property':
-            # @classmethod
-            # def func6(cls, _self): ...
-            func = lambda x: setattr(_cls, x, lambda *args, **kwargs: property(getattr(cls, x)(*args, **kwargs)))
-        elif mode == 'classmethod2classmethod':
-            # @classmethod
-            # def func7(cls, _cls, *args, **kwargs): ...
-            func = lambda x: setattr(_cls, x, lambda *args, **kwargs: classmethod(getattr(cls, x)(*args, **kwargs)))
-        elif mode == 'classmethod2classproperty':
-            # @classmethod
-            # def func8(cls, _cls): ...
-            func = lambda x: setattr(_cls, x, lambda *args, **kwargs: classproperty(getattr(cls, x)(*args, **kwargs)))
-        elif mode == 'staticmethod2modulefunc':
-            # @staticmethod
-            # def func9(*args, **kwargs): ...
-            func = lambda x: setattr(_cls, x, getattr(cls, x))
-        elif mode == 'classmethod2modulefunc':
-            # @classmethod
-            # def func10(cls, *args, **kwargs): ...
-            func = lambda x: setattr(_cls, x, lambda *args, **kwargs: getattr(cls, x)(*args, **kwargs))
-        # 理论上还有'2moduleproperty'的转换模式
-        #   但这个很容易引起歧义，是应该存一个数值，还是动态计算？
-        #   如果是动态计算，可以使用modulefunc的机制显式执行，更不容易引起混乱。
-        #   从这个分析来看，是不需要实现'2moduleproperty'的绑定体系的。py标准语法本来也就没有module @property的概念。
-        else:
-            raise ValueError
-
         for name in set(names):
-            func(name)
+            cvt(cls, _cls, name)
 
     @classmethod
     def enchant(cls):
         raise NotImplementedError
+
+
+def check_install_package(package, speccal_install_name=None, *, user=False):
+    """ https://stackoverflow.com/questions/12332975/installing-python-module-within-code
+
+    TODO 不知道频繁调用这个，会不会太影响性能，可以想想怎么提速优化？
+    注意不要加@RunOnlyOnce，亲测速度会更慢三倍
+
+    警告: 不要在频繁调用的底层函数里使用 check_install_package
+        如果是module级别的还好，调几次其实性能影响微乎其微
+        但在频繁调用的函数里使用，每百万次还是要额外的0.5秒开销的
+    """
+    try:
+        __import__(package)
+    except ModuleNotFoundError:
+        cmds = [sys.executable, "-m", "pip", "install"]
+        if user: cmds.append('--user')
+        cmds.append(speccal_install_name if speccal_install_name else package)
+        subprocess.check_call(cmds)
+
+
+def limit_call_number(limit=1, object_method=False):
+    """ 限制装饰的函数对象，最多执行几次 """
+
+    def decorator(func):
+        counter = defaultdict(int) if object_method else 0
+
+        def wrapper(*args, **kwargs):
+            nonlocal counter
+
+            if object_method:
+                id_ = id(args[0])
+                counter[id_] += 1
+                cnt = counter[id_]
+            else:
+                counter += 1
+                cnt = counter
+
+            if cnt <= limit:
+                return func(*args, **kwargs)
+
+        return wrapper
+
+    return decorator
